﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Text;
using System.Web.Mvc;
using aliyun_api_gateway_sdk.Constant;
using aliyun_api_gateway_sdk.Util;
using aspnetDemo.Models;
using Newtonsoft.Json;

namespace aspnetDemo.Controllers
{
    public class HomeController : Controller
    {
        private static string appKey = Environment.GetEnvironmentVariable("IOT.HOSTING.APPKEY");
        private static string appSecret = Environment.GetEnvironmentVariable("IOT.HOSTING.APPSECRET");
        private static string host = Environment.GetEnvironmentVariable("iot.hosting.api.domain");
        private static string redirectUrl = string.Empty;
        private static string currentAuthCode = string.Empty;
        private static OAuthResult<GetAccessCodeResult> lastRequestResult = null;

        /// <summary>
        /// 主页
        /// </summary>
        /// <param name="code">oauthCode参数</param>
        /// <returns>ActionResult</returns>
        public ActionResult Index(string code)
        {
            if (!string.IsNullOrWhiteSpace(code))
            {
                //如果有code参数，说明是登录iot平台成功并跳转到这里
                //实现免登逻辑放这里，比如：
                //1、根据GetUserInfoByAccessToken接口获取用户信息并判断该用户是否合法
                //2、使用随机或生成固定密码实现免登，或者生成一个一段时间内有效的签名sign实现免登
                currentAuthCode = code;
                return View("OAuthIndex");
            }
            return View();
        }

        public ActionResult OAuthIndex()
        {
            return View();
        }

        /// <summary>
        /// 拼接iot平台验证url
        /// </summary>
        /// <returns>ActionResult</returns>
        [HttpGet]
        public ActionResult OAuthVerification()
        {
            redirectUrl = $@"http://{Request.Headers.Get("Host")}";
            var url = $"https://account.iot.aliyun.com/oauth2/auth?client_id={appKey}&redirect_uri={redirectUrl}&response_type=code";
            return new RedirectResult(url, false);
        }

        /// <summary>
        ///  调用获取用户信息接口
        /// </summary>
        /// <param name="path">接口请求路径</param>
        /// <returns>请求结果Json字符串</returns>
        [HttpPost]
        public string GetUserInfoByAccessToken(string path)
        {
            var responseStr = SendRequest(path, new { lastRequestResult.data.access_token }, new { apiVer = "1.0.2" });
            return responseStr;
        }

        /// <summary>
        /// 调用刷新token令牌接口
        /// </summary>
        /// <param name="path">接口请求路径</param>
        /// <returns>请求结果Json字符串</returns>
        [HttpPost]
        public string RefreshToken(string path)
        {
            var responseStr = SendRequest(path, new { lastRequestResult.data.refresh_token }, new { apiVer = "1.0.1" });
            var result = JsonConvert.DeserializeObject<OAuthResult<RefreshTokenResult>>(responseStr);
            lastRequestResult.data.refresh_token = result.data.refresh_token;
            return responseStr;
        }

        /// <summary>
        /// 调用GetAccessCode接口，根据oauthcode换取accesscode
        /// </summary>
        /// <param name="path">接口请求路径</param>
        /// <returns>接口请求路径</returns>
        [HttpPost]
        public string GetAccessCode(string path)
        {
            try
            {
                if (null != lastRequestResult)
                {
                    return JsonConvert.SerializeObject(lastRequestResult);
                }

                var @params = new
                {
                    @code = currentAuthCode,
                    redirect_uri = redirectUrl,
                    client_id = appKey,
                    grant_type = "authorization_code"
                };
                var responseStr = SendRequest(path, @params, new { apiVer = "1.0.1" });
                lastRequestResult = JsonConvert.DeserializeObject<OAuthResult<GetAccessCodeResult>>(responseStr);
                return responseStr;
            }
            catch (Exception ex)
            {
                return ex.Message;
            }
        }

        /// <summary>
        /// 根据参数发送接口调用请求
        /// </summary>
        /// <returns>接口请求结果，一般是Json字符串格式</returns>
        /// <param name="requestPath">接口名称</param>
        /// <param name="params">调用接口需要的参数</param>
        /// <param name="apiVersion">接口版本信息</param>
        string SendRequest(string requestPath, object @params, object apiVersion)
        {
            var bobyContent = JsonConvert.SerializeObject(new
            {
                request= apiVersion,
                id = Guid.NewGuid().ToString(),
                @params,
                version = "1.0"
            });
            var headers = new Dictionary<string, string>();
            var querys = new Dictionary<string, string>();
            var bodys = new Dictionary<string, string>();
            var signHeader = new List<String>();

            //设定Content-Type，根据服务器端接受的值来设置¸
            headers.Add(HttpHeader.HTTP_HEADER_CONTENT_TYPE, ContentType.CONTENT_TYPE_STREAM);
            //设定Accept，根据服务器端接受的值来设置
            headers.Add(HttpHeader.HTTP_HEADER_ACCEPT, ContentType.CONTENT_TYPE_JSON);

            //注意：如果有非Form形式数据(body中只有value，没有key)；如果body中是key/value形式数据，不要指定此行
            // headers.Add(HttpHeader.HTTP_HEADER_CONTENT_MD5, MessageDigestUtil.Base64AndMD5(Encoding.UTF8.GetBytes(bobyContent)));

            //注意：业务body部分
            bodys.Add("", bobyContent);

            //指定参与签名的header
            signHeader.Add(SystemHeader.X_CA_TIMESTAMP);

            using (HttpWebResponse response = HttpUtil.HttpPost(host, requestPath, appKey, appSecret, 30000, headers, querys, bodys, signHeader))
            {
                var st = response.GetResponseStream();
                var reader = new StreamReader(st, Encoding.GetEncoding("utf-8"));
                // 通过reader.ReadToEnd获取返回结果，正常情况下结果是一个json string
                var responseStr = reader.ReadToEnd();
                return responseStr;
            }
        }
    }
}


